@@ -183,7 +183,7 @@ static void unref_loop (GMainLoop *loop)
 
 #define DBUS_TYPE_G_OBJECT_PATH_ARRAY (dbus_g_type_get_collection ("GPtrArray", DBUS_TYPE_G_OBJECT_PATH))
 
-static DBusGProxy *open_device(pam_handle_t *pamh, DBusGConnection *connection, DBusGProxy *manager, const char *username, gboolean *has_multiple_devices)
+static DBusGProxy *open_device(pam_handle_t *pamh, DBusGConnection *connection, DBusGProxy *manager, gboolean *has_multiple_devices)
 {
 	GError *error = NULL;
 	const char *path;
@@ -217,13 +217,6 @@ static DBusGProxy *open_device(pam_handle_t *pamh, DBusGConnection *connection,
 					path,
 					"net.reactivated.Fprint.Device");
 
-	if (!dbus_g_proxy_call (dev, "Claim", &error, G_TYPE_STRING, username, G_TYPE_INVALID, G_TYPE_INVALID)) {
-		D(pamh, "failed to claim device '%s': %s\n", path, error->message);
-		g_error_free (error);
-		g_object_unref (dev);
-		dev = NULL;
-	}
-
 	g_ptr_array_free (paths_array, TRUE);
 
 	return dev;
@@ -383,6 +376,26 @@ static int do_verify(GMainLoop *loop, pam_handle_t *pamh, DBusGProxy *dev, gbool
 	return ret;
 }
 
+static gboolean user_has_prints(DBusGProxy *dev, const char *username)
+{
+	char **fingers;
+	gboolean have_prints;
+
+	if (!dbus_g_proxy_call (dev, "ListEnrolledFingers", NULL,
+				G_TYPE_STRING, username, G_TYPE_INVALID,
+				G_TYPE_STRV, &fingers, G_TYPE_INVALID)) {
+		/* If ListEnrolledFingers fails then verification should
+		 * also fail (both use the same underlying call), so we
+		 * report FALSE here and bail out early.  */
+		return FALSE;
+	}
+
+	have_prints = fingers != NULL && g_strv_length (fingers) > 0;
+	g_strfreev (fingers);
+
+	return have_prints;
+}
+
 static void release_device(pam_handle_t *pamh, DBusGProxy *dev)
 {
 	GError *error = NULL;
@@ -392,30 +405,52 @@ static void release_device(pam_handle_t *pamh, DBusGProxy *dev)
 	}
 }
 
+static gboolean claim_device(pam_handle_t *pamh, DBusGProxy *dev, const char *username)
+{
+	GError *error = NULL;
+
+	if (!dbus_g_proxy_call (dev, "Claim", &error, G_TYPE_STRING, username, G_TYPE_INVALID, G_TYPE_INVALID)) {
+		D(pamh, "failed to claim device %s\n", error->message);
+		g_error_free (error);
+		return FALSE;
+	}
+
+	return TRUE;
+}
+
 static int do_auth(pam_handle_t *pamh, const char *username)
 {
 	DBusGProxy *manager;
 	DBusGConnection *connection;
 	DBusGProxy *dev;
 	GMainLoop *loop;
+	gboolean have_prints;
 	gboolean has_multiple_devices;
-	int ret;
+	int ret = PAM_AUTHINFO_UNAVAIL;
 
 	manager = create_manager (pamh, &connection, &loop);
 	if (manager == NULL)
 		return PAM_AUTHINFO_UNAVAIL;
 
-	dev = open_device(pamh, connection, manager, username, &has_multiple_devices);
+	dev = open_device(pamh, connection, manager, &has_multiple_devices);
 	g_object_unref (manager);
 	if (!dev) {
 		unref_loop (loop);
 		close_and_unref (connection);
 		return PAM_AUTHINFO_UNAVAIL;
 	}
-	ret = do_verify(loop, pamh, dev, has_multiple_devices);
+
+	have_prints = user_has_prints(dev, username);
+	D(pamh, "prints registered: %s\n", have_prints ? "yes" : "no");
+
+	if (have_prints) {
+		if (claim_device (pamh, dev, username)) {
+			ret = do_verify (loop, pamh, dev, has_multiple_devices);
+			release_device (pamh, dev);
+		}
+	}
 
 	unref_loop (loop);
-	release_device(pamh, dev);
 	g_object_unref (dev);
 	close_and_unref (connection);
 
@@ -433,7 +468,9 @@ PAM_EXTERN int pam_sm_authenticate(pam_handle_t *pamh, int flags, int argc,
 	bindtextdomain (GETTEXT_PACKAGE, LOCALEDIR);
 	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
 
-	g_type_init ();
+#if !GLIB_CHECK_VERSION (2, 36, 0)
+	g_type_init();
+#endif
 
 	dbus_g_object_register_marshaller (fprintd_marshal_VOID__STRING_BOOLEAN,
 					   G_TYPE_NONE, G_TYPE_STRING, G_TYPE_BOOLEAN, G_TYPE_INVALID);
