@@ -558,7 +558,12 @@ _fprint_device_client_vanished (GDBusConnection *connection,
 
 		/* Close the claimed device as well */
 		if (priv->dev) {
-			fp_async_dev_close (priv->dev, action_stop_cb, &done);
+			struct fp_dev *dev;
+
+			dev = priv->dev;
+			priv->dev = NULL;
+
+			fp_async_dev_close (dev, action_stop_cb, &done);
 			while (done == FALSE)
 				g_main_context_iteration (NULL, TRUE);
 		}
@@ -601,7 +606,7 @@ static void dev_open_cb(struct fp_dev *dev, int status, void *user_data)
 	FprintDevicePrivate *priv = DEVICE_GET_PRIVATE(rdev);
 	struct session_data *session = priv->session;
 
-	g_message("device %d claim status %d", priv->id, status);
+	g_debug("device %d claim status %d", priv->id, status);
 
 	if (status != 0) {
 		GError *error = NULL;
@@ -670,7 +675,7 @@ static void fprint_device_claim(FprintDevice *rdev,
 	priv->username = user;
 	priv->sender = sender;
 
-	g_message ("user '%s' claiming the device: %d", priv->username, priv->id);
+	g_debug ("user '%s' claiming the device: %d", priv->username, priv->id);
 
 	priv->session = g_slice_new0(struct session_data);
 	priv->session->context_claim_device = context;
@@ -709,7 +714,7 @@ static void dev_close_cb(struct fp_dev *dev, void *user_data)
 	g_free (priv->username);
 	priv->username = NULL;
 
-	g_message("released device %d", priv->id);
+	g_debug("released device %d", priv->id);
 	dbus_g_method_return(context);
 }
 
@@ -737,8 +742,13 @@ static void fprint_device_release(FprintDevice *rdev,
 	}
 
 	session->context_release_device = context;
-	if (priv->dev)
-		fp_async_dev_close(priv->dev, dev_close_cb, rdev);
+	if (priv->dev) {
+		struct fp_dev *dev;
+
+		dev = priv->dev;
+		priv->dev = NULL;
+		fp_async_dev_close(dev, dev_close_cb, rdev);
+	}
 }
 
 static void verify_cb(struct fp_dev *dev, int r, struct fp_img *img,
@@ -751,7 +761,7 @@ static void verify_cb(struct fp_dev *dev, int r, struct fp_img *img,
 	if (priv->action_done != FALSE)
 		return;
 
-	g_message("verify_cb: result %s (%d)", name, r);
+	g_debug("verify_cb: result %s (%d)", name, r);
 
 	if (r == FP_VERIFY_NO_MATCH || r == FP_VERIFY_MATCH || r < 0)
 		priv->action_done = TRUE;
@@ -775,7 +785,7 @@ static void identify_cb(struct fp_dev *dev, int r,
 	if (priv->action_done != FALSE)
 		return;
 
-	g_message("identify_cb: result %s (%d)", name, r);
+	g_debug("identify_cb: result %s (%d)", name, r);
 
 	if (r == FP_VERIFY_NO_MATCH || r == FP_VERIFY_MATCH || r < 0)
 		priv->action_done = TRUE;
@@ -846,7 +856,7 @@ static void fprint_device_verify_start(FprintDevice *rdev,
 			array = g_ptr_array_new ();
 
 			for (l = prints; l != NULL; l = l->next) {
-				g_message ("adding finger %d to the gallery", GPOINTER_TO_INT (l->data));
+				g_debug ("adding finger %d to the gallery", GPOINTER_TO_INT (l->data));
 				r = store.print_data_load(priv->dev, GPOINTER_TO_INT (l->data),
 							  &data, priv->username);
 				if (r == 0)
@@ -876,12 +886,12 @@ static void fprint_device_verify_start(FprintDevice *rdev,
 		}
 		priv->current_action = ACTION_IDENTIFY;
 
-		g_message ("start identification device %d", priv->id);
+		g_debug ("start identification device %d", priv->id);
 		r = fp_async_identify_start (priv->dev, gallery, identify_cb, rdev);
 	} else {
 		priv->current_action = ACTION_VERIFY;
 
-		g_message("start verification device %d finger %d", priv->id, finger_num);
+		g_debug("start verification device %d finger %d", priv->id, finger_num);
 
 		r = store.print_data_load(priv->dev, (enum fp_finger)finger_num, 
 					  &data, priv->username);
@@ -1007,7 +1017,7 @@ static void enroll_stage_cb(struct fp_dev *dev, int result,
 	if (priv->action_done != FALSE)
 		return;
 
-	g_message("enroll_stage_cb: result %d", result);
+	g_debug("enroll_stage_cb: result %d", result);
 	if (result == FP_ENROLL_COMPLETE) {
 		r = store.print_data_save(print, session->enroll_finger, priv->username);
 		if (r < 0)
@@ -1066,7 +1076,7 @@ static void fprint_device_enroll_start(FprintDevice *rdev,
 		return;
 	}
 
-	g_message("start enrollment device %d finger %d", priv->id, finger_num);
+	g_debug("start enrollment device %d finger %d", priv->id, finger_num);
 	session->enroll_finger = finger_num;
 	priv->action_done = FALSE;
 	
